"""Config flow to configure the LK3 integration."""
import logging
from typing import Any, Dict, Optional

import voluptuous as vol
from homeassistant.config_entries import CONN_CLASS_LOCAL_POLL, ConfigFlow
from homeassistant.const import (CONF_HOST, CONF_MAC, CONF_NAME, CONF_PASSWORD,
                                 CONF_PORT, CONF_USERNAME)
from homeassistant.helpers.aiohttp_client import async_get_clientsession
from homeassistant.helpers.typing import ConfigType

from .const import DEFAULT_UPDATE_INTERVAL, DOMAIN, UPDATE_INTERVAL
from .lk3api import (LK3AuthenticationError, LK3ConnectionError, LK3Reader,
                     LK3VersionError)

_LOGGER = logging.getLogger(__name__)


class LK3FlowHandler(ConfigFlow, domain=DOMAIN):
    """Handle a LK3 config flow."""

    VERSION = 1
    CONNECTION_CLASS = CONN_CLASS_LOCAL_POLL

    data_schema = vol.Schema({
        vol.Required(CONF_NAME): str,
        vol.Required(CONF_HOST): str,
        vol.Optional(CONF_USERNAME, default=''): str,
        vol.Optional(CONF_PASSWORD, default=''): str,
        vol.Optional(UPDATE_INTERVAL, default=DEFAULT_UPDATE_INTERVAL): vol.All(int, vol.Range(min=1)),
        vol.Optional(CONF_PORT, default=80): int,
    })

    async def async_step_user(
            self, user_input: Optional[ConfigType] = None,
    ) -> Dict[str, Any]:
        """Handle a flow initiated by the user."""
        errors = {}
        if user_input is not None:
            # Try to connect with device to verify config
            session = async_get_clientsession(self.hass)
            reader = LK3Reader(
                user_input[CONF_HOST],
                user_input[CONF_USERNAME],
                user_input[CONF_PASSWORD],
                user_input[CONF_PORT],
                session=session,
            )
            try:
                device = await reader.update()
            except LK3ConnectionError:
                errors['base'] = 'cannot_connect'
            except LK3AuthenticationError:
                errors[CONF_USERNAME] = 'wrong_credentials'
                errors[CONF_PASSWORD] = 'wrong_credentials'
            except LK3VersionError:
                errors['base'] = 'wrong_version'
            else:
                mac_address = device.info.mac_address
                for entry in self.hass.config_entries.async_entries(DOMAIN):
                    if entry.data[CONF_HOST] == user_input[CONF_HOST]:
                        return self.async_abort(reason='already_configured')
                    if entry.data[CONF_MAC] == mac_address:
                        return self.async_abort(reason='already_configured')
                    if entry.data[CONF_NAME] == user_input[CONF_NAME]:
                        errors[CONF_NAME] = 'name_exists'
                        break

            if not errors:
                data = user_input.copy()
                data.update({CONF_MAC: mac_address})
                return self.async_create_entry(
                    title=user_input[CONF_NAME],
                    data=data,
                )
        return self.async_show_form(
            step_id='user',
            data_schema=self.data_schema,
            errors=errors,
        )
